@@ -46,6 +46,7 @@
 
 #include "include/rbd_types.h"
 #include "common/TextTable.h"
+#include "include/util.h"
 
 #if defined(__linux__)
 #include <linux/fs.h>
@@ -82,9 +83,11 @@ void usage()
 "  resize --size <MB> <image-name>             resize (expand or contract) image\n"
 "  rm <image-name>                             delete an image\n"
 "  export <image-name> <path>                  export image to file\n"
+"                                              \"-\" for stdout\n"
 "  import <path> <image-name>                  import image from file\n"
-"                                              (dest defaults)\n"
-"                                              as the filename part of file)\n"
+"                                              (dest defaults\n"
+"                                               as the filename part of file)\n"
+"                                              \"-\" for stdin\n"
 "  (cp | copy) <src> <dest>                    copy src image to dest\n"
 "  (mv | rename) <src> <dest>                  rename src image to dest\n"
 "  snap ls <image-name>                        dump list of image snapshots\n"
@@ -104,6 +107,7 @@ void usage()
 "  lock list <image-name>                      show locks held on an image\n"
 "  lock add <image-name> <id> [--shared <tag>] take a lock called id on an image\n"
 "  lock remove <image-name> <id> <locker>      release a lock on an image\n"
+"  bench-write <image-name> --io-size <bytes> --io-threads <num> --io-total <bytes>\n"
 "\n"
 "<image-name>, <snap-name> are [pool/]name[@snap], or you may specify\n"
 "individual pieces of names with -p/--pool, --image, and/or --snap.\n"
@@ -132,6 +136,11 @@ static string feature_str(uint64_t features)
 
   if (features & RBD_FEATURE_LAYERING)
     s += "layering";
+  if (features & RBD_FEATURE_STRIPINGV2) {
+    if (s.size())
+      s += ", ";
+    s += "striping";
+  }
   return s;
 }
 
@@ -191,7 +200,19 @@ static int do_list(librbd::RBD &rbd, librados::IoCtx& io_ctx, bool lflag)
     librbd::image_info_t info;
     librbd::Image im;
 
-    rbd.open(io_ctx, im, i->c_str());
+    r = rbd.open_read_only(io_ctx, im, i->c_str(), NULL);
+    // image might disappear between rbd.list() and rbd.open(); ignore
+    // that, warn about other possible errors (EPERM, say, for opening
+    // an old-format image, because you need execute permission for the
+    // class method)
+    if (r < 0) {
+      if (r != -ENOENT) {
+	cerr << "rbd: error opening " << *i << ": " << cpp_strerror(r)
+	     << std::endl;
+      }
+      // in any event, continue to next image
+      continue;
+    }
 
     // handle second-nth trips through loop
     parent.clear();
@@ -255,16 +276,27 @@ static int do_list(librbd::RBD &rbd, librados::IoCtx& io_ctx, bool lflag)
 
 static int do_create(librbd::RBD &rbd, librados::IoCtx& io_ctx,
 		     const char *imgname, uint64_t size, int *order,
-		     int format, uint64_t features)
+		     int format, uint64_t features,
+		     uint64_t stripe_unit, uint64_t stripe_count)
 {
   int r;
-  if (features == 0)
-    features = RBD_FEATURES_ALL;
 
-  if (format == 1)
+  if (format == 1) {
+    // weird striping not allowed with format 1!
+    if ((stripe_unit || stripe_count) &&
+	(stripe_unit != (1ull << *order) && stripe_count != 1)) {
+      cerr << "non-default striping not allowed with format 1; use --format 2" << std::endl;
+      return -EINVAL;
+    }
     r = rbd.create(io_ctx, imgname, size, order);
-  else
-    r = rbd.create2(io_ctx, imgname, size, features, order);
+  } else {
+    if (features == 0) {
+      features = RBD_FEATURE_LAYERING;
+    }
+    if (stripe_unit != (1ull << *order) && stripe_count != 1)
+      features |= RBD_FEATURE_STRIPINGV2;
+    r = rbd.create3(io_ctx, imgname, size, features, order, stripe_unit, stripe_count);
+  }
   if (r < 0)
     return r;
   return 0;
@@ -313,6 +345,7 @@ static int do_show_info(const char *imgname, librbd::Image& image,
   uint8_t old_format;
   uint64_t overlap, features;
   bool snap_protected;
+
   int r = image.stat(info, sizeof(info));
   if (r < 0)
     return r;
@@ -346,7 +379,6 @@ static int do_show_info(const char *imgname, librbd::Image& image,
        << std::endl
        << "\tformat: " << (old_format ? "1" : "2")
        << std::endl;
-
   if (!old_format) {
     cout << "\tfeatures: " << feature_str(features) << std::endl;
   }
@@ -365,6 +397,12 @@ static int do_show_info(const char *imgname, librbd::Image& image,
 	 << "@" << parent_snapname << std::endl;
     cout << "\toverlap: " << prettybyte_t(overlap) << std::endl;
   }
+
+  // striping info, if feature is set
+  if (features & RBD_FEATURE_STRIPINGV2) {
+    cout << "\tstripe unit: " << prettybyte_t(image.get_stripe_unit()) << std::endl
+	 << "\tstripe count: " << image.get_stripe_count() << std::endl;
+  }
   return 0;
 }
 
@@ -454,7 +492,11 @@ static int do_purge_snaps(librbd::Image& image)
   }
 
   for (size_t i = 0; i < snaps.size(); ++i) {
-    image.snap_remove(snaps[i].name.c_str());
+    r = image.snap_remove(snaps[i].name.c_str());
+    if (r < 0) {
+      pc.fail();
+      return r;
+    }
     pc.update_progress(i + 1, snaps.size());
   }
 
@@ -540,11 +582,109 @@ static int do_lock_remove(librbd::Image& image, const char *client,
   return image.break_lock(client, cookie);
 }
 
+static void rbd_bencher_completion(void *c, void *pc);
+
+struct rbd_bencher;
+
+struct rbd_bencher {
+  librbd::Image *image;
+  Mutex lock;
+  Cond cond;
+  int in_flight;
+
+  rbd_bencher(librbd::Image *i)
+    : image(i),
+      lock("rbd_bencher::lock"),
+      in_flight(0)
+  { }
+
+  bool start_write(int max, uint64_t off, uint64_t len, bufferlist& bl) {
+    Mutex::Locker l(lock);
+    if (in_flight >= max)
+      return false;
+    in_flight++;
+    librbd::RBD::AioCompletion *c = new librbd::RBD::AioCompletion((void *)this, rbd_bencher_completion);
+    image->aio_write(off, len, bl, c);
+    //cout << "start " << c << " at " << off << "~" << len << std::endl;
+    return true;
+  }
+
+  void wait_for(int max) {
+    Mutex::Locker l(lock);
+    while (in_flight > max) {
+      utime_t dur;
+      dur.set_from_double(.2);
+      cond.WaitInterval(g_ceph_context, lock, dur);
+    }
+  }
+
+};
+
+void rbd_bencher_completion(void *vc, void *pc)
+{
+  librbd::RBD::AioCompletion *c = (librbd::RBD::AioCompletion *)vc;
+  rbd_bencher *b = (rbd_bencher *)pc;
+  //cout << "complete " << c << std::endl;
+  b->lock.Lock();
+  b->in_flight--;
+  b->cond.Signal();
+  b->lock.Unlock();
+  c->release();
+}
+
+static int do_bench_write(librbd::Image& image, uint64_t io_size, uint64_t io_threads, uint64_t io_bytes)
+{
+  rbd_bencher b(&image);
+
+  cout << "bench-write "
+       << " io_size " << io_size
+       << " io_threads " << io_threads
+       << " bytes " << io_bytes
+       << std::endl;
+
+  bufferptr bp(io_size);
+  bp.zero();
+  bufferlist bl;
+  bl.push_back(bp);
+
+  utime_t start = ceph_clock_now(NULL);
+  utime_t last;
+  unsigned ios = 0;
+
+  printf("  SEC       OPS   OPS/SEC   BYTES/SEC\n");
+  uint64_t off;
+  for (off = 0; off < io_bytes; off += io_size) {
+    b.wait_for(io_threads - 1);
+    while (b.start_write(io_threads, off, io_size, bl))
+      ios++;
+
+    utime_t now = ceph_clock_now(NULL);
+    utime_t elapsed = now - start;
+    if (elapsed.sec() != last.sec()) {
+      printf("%5d  %8d  %8.2lf  %8.2lf\n", (int)elapsed, (int)(ios - io_threads),
+	     (double)(ios - io_threads) / elapsed,
+	     (double)(off - io_threads * io_size) / elapsed);
+      last = elapsed;
+    }
+  }
+  b.wait_for(0);
+
+  utime_t now = ceph_clock_now(NULL);
+  double elapsed = now - start;
+
+  printf("elapsed: %5d  ops: %8d  ops/sec: %8.2lf  bytes/sec: %8.2lf\n", (int)elapsed, ios,
+	     (double)ios / elapsed,
+	     (double)off / elapsed);
+
+  return 0;
+}
+
 struct ExportContext {
   int fd;
+  uint64_t totalsize;
   MyProgressContext pc;
 
-  ExportContext(int f) : fd(f), pc("Exporting image") {}
+  ExportContext(int f, uint64_t t) : fd(f), totalsize(t), pc("Exporting image") {}
 };
 
 static int export_read_cb(uint64_t ofs, size_t len, const char *buf, void *arg)
@@ -552,18 +692,51 @@ static int export_read_cb(uint64_t ofs, size_t len, const char *buf, void *arg)
   ssize_t ret;
   ExportContext *ec = (ExportContext *)arg;
   int fd = ec->fd;
+  static char *localbuf = NULL;
+  static size_t maplen = 0;
 
-  if (!buf) /* a hole */
-    return 0;
+  if (fd == 1) {
+    if (!buf) {
+      // can't seek stdout; need actual data to write
+      if (maplen < len) {
+	// never mapped, or need to map larger
+	int r;
+	if (localbuf != NULL){
+	  if ((r = munmap(localbuf, len)) < 0) {
+	    cerr << "rbd: error " << r << "munmap'ing buffer" << std::endl;
+	    return errno;
+	  }
+	}
+
+	maplen = len;
+	localbuf = (char *)mmap(NULL, maplen, PROT_READ,
+			        MAP_PRIVATE|MAP_ANONYMOUS, -1, 0);
+	if (localbuf == MAP_FAILED) {
+	  cerr << "rbd: MAP_FAILED mmap'ing buffer for zero writes"
+	       << std::endl;
+	  return -ENOMEM;
+	}
+      }
+      ret = write(fd, localbuf, len);
+    } else {
+      ret = write(fd, buf, len);
+    }
+  } else {		// not stdout
+    if (!buf || buf_is_zero(buf, len)) {
+      /* a hole */
+      return 0;
+    }
+
+    ret = lseek64(fd, ofs, SEEK_SET);
+    if (ret < 0)
+      return -errno;
+    ret = write(fd, buf, len);
+    ec->pc.update_progress(ofs, ec->totalsize);
+  }
 
-  ret = lseek64(fd, ofs, SEEK_SET);
-  if (ret < 0)
-    return -errno;
-  ret = write(fd, buf, len);
   if (ret < 0)
     return -errno;
 
-  cerr << "writing " << len << " bytes at ofs " << ofs << std::endl;
   return 0;
 }
 
@@ -577,16 +750,20 @@ static int do_export(librbd::Image& image, const char *path)
   if (r < 0)
     return r;
 
-  fd = open(path, O_WRONLY | O_CREAT | O_EXCL, 0644);
+  if (strcmp(path, "-") == 0)
+    fd = 1;
+  else
+    fd = open(path, O_WRONLY | O_CREAT | O_EXCL, 0644);
   if (fd < 0)
     return -errno;
 
-  ExportContext ec(fd);
+  ExportContext ec(fd, info.size);
   r = image.read_iterate(0, info.size, export_read_cb, (void *)&ec);
   if (r < 0)
     goto out;
 
-  r = ftruncate(fd, info.size);
+  if (fd != 1)
+    r = ftruncate(fd, info.size);
   if (r < 0)
     goto out;
 
@@ -635,9 +812,6 @@ static void set_pool_image_name(const char *orig_pool, const char *orig_img,
 {
   const char *sep;
 
-  if (orig_pool)
-    return;
-
   if (!orig_img)
     return;
 
@@ -658,6 +832,82 @@ done_img:
   update_snap_name(*new_img, snap);
 }
 
+static int do_import_from_stdin(librbd::RBD &rbd, librados::IoCtx& io_ctx,
+		                const char *imgname, int *order, int format,
+				int64_t features, int64_t size)
+{
+  int r;
+  // start with inherent block size; double as needed; fix when done
+  size_t cur_size = 1 << *order;
+
+  r = do_create(rbd, io_ctx, imgname, cur_size, order, format, features, 0, 0);
+  if (r < 0) {
+    cerr << "rbd: image creation failed" << std::endl;
+    return r;
+  }
+  librbd::Image image;
+  r = rbd.open(io_ctx, image, imgname);
+  if (r < 0) {
+    cerr << "rbd: failed to open image" << std::endl;
+    return r;
+  }
+
+  off_t image_pos = 0;
+
+  // try to fill whole imgblklen blocks for sparsification
+
+  size_t imgblklen = 1 << *order;
+  char *p = new char[imgblklen];
+  size_t reqlen = imgblklen;	// amount requested from read
+  ssize_t readlen;		// amount received from one read
+  size_t blklen = 0;		// amount accumulated from reads to fill blk
+
+  // loop body handles 0 return, as we may have a block to flush
+  while ((readlen = ::read(0, p + blklen, reqlen)) >= 0) {
+    blklen += readlen;
+    // if read was short, try again to fill the block before writing
+    if (readlen && ((size_t)readlen < reqlen)) {
+      reqlen -= readlen;
+      continue;
+    }
+
+    // resize output image by binary expansion as we go
+    if ((image_pos + (size_t)blklen) > cur_size) {
+      cur_size *= 2;
+      r = image.resize(cur_size);
+      if (r < 0) {
+	cerr << "rbd: can't resize image during import" << std::endl;
+	return r;
+      }
+    }
+
+    // write as much as we got; perhaps less than imgblklen
+    bufferlist bl(blklen);
+    bl.append(p, blklen);
+    // but skip writing zeros to create sparse images
+    if (!bl.is_zero()) {
+      r = image.write(image_pos, blklen, bl);
+      if (r < 0) {
+	cerr << "rbd: error writing to image block" << cpp_strerror(r) << std::endl;
+	return r;
+      }
+    }
+    // done with whole block, whether written or not 
+    image_pos += blklen;
+    // if read had returned 0, we're at EOF and should quit
+    if (readlen == 0)
+      break;
+    blklen = 0;
+    reqlen = imgblklen;
+  }
+  r = image.resize(image_pos);
+  if (r < 0) {
+    cerr << "rbd: final image resize failed" << std::endl;
+    return r;
+  }
+  return readlen;	// 0 or -error
+}
+
 static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
 		     const char *imgname, int *order, const char *path,
 		     int format, uint64_t features, int64_t size)
@@ -667,11 +917,16 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
   struct fiemap *fiemap;
   MyProgressContext pc("Importing image");
 
-  if (! strcmp(path, "-")) {
-    fd = 0;
-  } else {
-    fd = open(path, O_RDONLY);
-  }
+  assert(imgname);
+
+  // default order as usual
+  if (*order == 0)
+    *order = 22;
+
+  if (!strcmp(path, "-"))
+    return do_import_from_stdin(rbd, io_ctx, imgname, order, format,
+				features, size);
+  fd = open(path, O_RDONLY);
 
   if (fd < 0) {
     r = -errno;
@@ -699,9 +954,7 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
     }
   }
 
-  assert(imgname);
-
-  r = do_create(rbd, io_ctx, imgname, size, order, format, features);
+  r = do_create(rbd, io_ctx, imgname, size, order, format, features, 0, 0);
   if (r < 0) {
     cerr << "rbd: image creation failed" << std::endl;
     close(fd);
@@ -714,6 +967,7 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
     close(fd);
     return r;
   }
+
   fsync(fd); /* flush it first, otherwise extents information might not have been flushed yet */
   fiemap = read_fiemap(fd);
   if (fiemap && !fiemap->fm_mapped_extents) {
@@ -744,6 +998,7 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
   while (extent < fiemap->fm_mapped_extents) {
     off_t file_pos, end_ofs;
     size_t extent_len = 0;
+    size_t read_len = 1ULL << *order;
 
     file_pos = fiemap->fm_extents[extent].fe_logical; /* position within the file we're reading */
 
@@ -762,11 +1017,10 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
 
     } while (end_ofs == (off_t)fiemap->fm_extents[extent].fe_logical);
 
-#define READ_BLOCK_LEN (4 * 1024 * 1024)
     uint64_t left = end_ofs - file_pos;
     while (left) {
       pc.update_progress(file_pos, size);
-      uint64_t cur_seg = (left < READ_BLOCK_LEN ? left : READ_BLOCK_LEN);
+      uint64_t cur_seg = (left < read_len ? left : read_len);
       while (cur_seg) {
         bufferptr p(cur_seg);
 	ssize_t rval;
@@ -792,7 +1046,7 @@ static int do_import(librbd::RBD &rbd, librados::IoCtx& io_ctx,
           r = -ENOMEM;
           goto done;
         }
-        r = image.aio_write(file_pos, len, bl, completion);
+	r = image.aio_write(file_pos, len, bl, completion);
         if (r < 0)
           goto done;
 	completion->wait_for_complete();
@@ -1174,6 +1428,7 @@ enum {
   OPT_LOCK_LIST,
   OPT_LOCK_ADD,
   OPT_LOCK_REMOVE,
+  OPT_BENCH_WRITE,
 };
 
 static int get_cmd(const char *cmd, bool snapcmd, bool lockcmd)
@@ -1214,6 +1469,8 @@ static int get_cmd(const char *cmd, bool snapcmd, bool lockcmd)
       return OPT_SHOWMAPPED;
     if (strcmp(cmd, "unmap") == 0)
       return OPT_UNMAP;
+    if (strcmp(cmd, "bench-write") == 0)
+      return OPT_BENCH_WRITE;
   } else if (snapcmd) {
     if (strcmp(cmd, "create") == 0 ||
         strcmp(cmd, "add") == 0)
@@ -1267,6 +1524,8 @@ static bool set_conf_param(const char *param, const char **var1,
   return true;
 }
 
+bool size_set;
+
 int main(int argc, const char **argv)
 {
   librados::Rados rados;
@@ -1293,6 +1552,8 @@ int main(int argc, const char **argv)
     *devpath = NULL, *lock_cookie = NULL, *lock_client = NULL,
     *lock_tag = NULL;
   bool lflag = false;
+  long long stripe_unit = 0, stripe_count = 0;
+  long long bench_io_size = 4096, bench_io_threads = 16, bench_bytes = 1 << 30;
 
   std::string val;
   std::ostringstream err;
@@ -1330,14 +1591,25 @@ int main(int argc, const char **argv)
 	cerr << "rbd: " << err.str() << std::endl;
 	return EXIT_FAILURE;
       }
+      if (sizell < 0) {
+	cerr << "rbd: size must be >= 0" << std::endl;
+	return EXIT_FAILURE;
+      }
       size = sizell << 20;   // bytes to MB
+      size_set = true;
     } else if (ceph_argparse_flag(args, i, "-l", "--long", (char*)NULL)) {
       lflag = true;
+    } else if (ceph_argparse_withlonglong(args, i, &stripe_unit, &err, "--stripe-unit", (char*)NULL)) {
+    } else if (ceph_argparse_withlonglong(args, i, &stripe_count, &err, "--stripe-count", (char*)NULL)) {
     } else if (ceph_argparse_withint(args, i, &order, &err, "--order", (char*)NULL)) {
       if (!err.str().empty()) {
 	cerr << "rbd: " << err.str() << std::endl;
 	return EXIT_FAILURE;
       }
+    } else if (ceph_argparse_withlonglong(args, i, &bench_io_size, &err, "--io-size", (char*)NULL)) {
+    } else if (ceph_argparse_withlonglong(args, i, &bench_io_threads, &err, "--io-threads", (char*)NULL)) {
+    } else if (ceph_argparse_withlonglong(args, i, &bench_bytes, &err, "--io-total", (char*)NULL)) {
+    } else if (ceph_argparse_withlonglong(args, i, &stripe_count, &err, "--stripe-count", (char*)NULL)) {
     } else if (ceph_argparse_witharg(args, i, &val, "--path", (char*)NULL)) {
       path = strdup(val.c_str());
     } else if (ceph_argparse_witharg(args, i, &val, "--dest", (char*)NULL)) {
@@ -1410,6 +1682,7 @@ if (!set_conf_param(v, p1, p2, p3)) { \
       case OPT_SNAP_UNPROTECT:
       case OPT_WATCH:
       case OPT_MAP:
+      case OPT_BENCH_WRITE:
       case OPT_LOCK_LIST:
 	SET_CONF_PARAM(v, &imgname, NULL, NULL);
 	break;
@@ -1472,6 +1745,7 @@ if (!set_conf_param(v, p1, p2, p3)) { \
     destname = imgname;
     if (!destname)
       destname = imgname_from_path(path);
+    imgname = NULL;
   }
 
   if (opt_cmd != OPT_LOCK_ADD && lock_tag) {
@@ -1521,10 +1795,24 @@ if (!set_conf_param(v, p1, p2, p3)) { \
 
   set_pool_image_name(dest_poolname, destname, (char **)&dest_poolname, (char **)&destname, (char **)&dest_snapname);
 
+  if (opt_cmd == OPT_IMPORT) {
+    if (poolname && dest_poolname) {
+      cerr << "rbd: source and destination pool both specified" << std::endl;
+      return EXIT_FAILURE;
+    }
+    if (imgname && destname) {
+      cerr << "rbd: source and destination image both specified" << std::endl;
+      return EXIT_FAILURE;
+    }
+    if (poolname)
+      dest_poolname = poolname;
+  }
+
   if (!poolname)
     poolname = "rbd";
+
   if (!dest_poolname)
-    dest_poolname = poolname;
+    dest_poolname = "rbd";
 
   if (opt_cmd == OPT_EXPORT && !path)
     path = imgname;
@@ -1576,16 +1864,23 @@ if (!set_conf_param(v, p1, p2, p3)) { \
   }
 
   if (imgname && talk_to_cluster &&
-      (opt_cmd == OPT_RESIZE || opt_cmd == OPT_INFO ||
-       opt_cmd == OPT_SNAP_LIST || opt_cmd == OPT_SNAP_CREATE ||
+      (opt_cmd == OPT_RESIZE || opt_cmd == OPT_SNAP_CREATE ||
        opt_cmd == OPT_SNAP_ROLLBACK || opt_cmd == OPT_SNAP_REMOVE ||
-       opt_cmd == OPT_SNAP_PURGE || opt_cmd == OPT_EXPORT ||
-       opt_cmd == OPT_SNAP_PROTECT || opt_cmd == OPT_SNAP_UNPROTECT ||
-       opt_cmd == OPT_WATCH || opt_cmd == OPT_COPY ||
-       opt_cmd == OPT_FLATTEN || opt_cmd == OPT_CHILDREN ||
-       opt_cmd == OPT_LOCK_LIST || opt_cmd == OPT_LOCK_ADD ||
-       opt_cmd == OPT_LOCK_REMOVE)) {
-    r = rbd.open(io_ctx, image, imgname);
+       opt_cmd == OPT_SNAP_PURGE || opt_cmd == OPT_SNAP_PROTECT ||
+       opt_cmd == OPT_SNAP_UNPROTECT || opt_cmd == OPT_WATCH ||
+       opt_cmd == OPT_FLATTEN || opt_cmd == OPT_LOCK_ADD ||
+       opt_cmd == OPT_LOCK_REMOVE || opt_cmd == OPT_BENCH_WRITE ||
+       opt_cmd == OPT_INFO || opt_cmd == OPT_SNAP_LIST ||
+       opt_cmd == OPT_EXPORT || opt_cmd == OPT_COPY ||
+       opt_cmd == OPT_CHILDREN || opt_cmd == OPT_LOCK_LIST)) {
+
+    if (opt_cmd == OPT_INFO || opt_cmd == OPT_SNAP_LIST ||
+	opt_cmd == OPT_EXPORT || opt_cmd == OPT_COPY ||
+	opt_cmd == OPT_CHILDREN || opt_cmd == OPT_LOCK_LIST) {
+      r = rbd.open_read_only(io_ctx, image, imgname, NULL);
+    } else {
+      r = rbd.open(io_ctx, image, imgname);
+    }
     if (r < 0) {
       cerr << "rbd: error opening image " << imgname << ": "
 	   << cpp_strerror(-r) << std::endl;
@@ -1613,6 +1908,13 @@ if (!set_conf_param(v, p1, p2, p3)) { \
     }
   }
 
+  if (opt_cmd == OPT_CREATE || opt_cmd == OPT_RESIZE) {
+    if (!size_set) {
+      cerr << "rbd: must specify --size <MB>" << std::endl;
+      return EXIT_FAILURE;
+    }
+  }
+
   switch (opt_cmd) {
   case OPT_LIST:
     r = do_list(rbd, io_ctx, lflag);
@@ -1630,17 +1932,17 @@ if (!set_conf_param(v, p1, p2, p3)) { \
     break;
 
   case OPT_CREATE:
-    if (!size) {
-      cerr << "rbd: must specify size in MB to create an rbd image"
-	   << std::endl;
-      return EXIT_FAILURE;
-    }
     if (order && (order < 12 || order > 25)) {
       cerr << "rbd: order must be between 12 (4 KB) and 25 (32 MB)"
 	   << std::endl;
       return EXIT_FAILURE;
     }
-    r = do_create(rbd, io_ctx, imgname, size, &order, format, features);
+    if ((stripe_unit && !stripe_count) || (!stripe_unit && stripe_count)) {
+      cerr << "must specify both (or neither) of stripe-unit and stripe-count" << std::endl;
+      usage();
+      return EXIT_FAILURE;
+    }
+    r = do_create(rbd, io_ctx, imgname, size, &order, format, features, stripe_unit, stripe_count);
     if (r < 0) {
       cerr << "rbd: create error: " << cpp_strerror(-r) << std::endl;
       return EXIT_FAILURE;
@@ -1913,6 +2215,14 @@ if (!set_conf_param(v, p1, p2, p3)) { \
       return EXIT_FAILURE;
     }
     break;
+
+  case OPT_BENCH_WRITE:
+    r = do_bench_write(image, bench_io_size, bench_io_threads, bench_bytes);
+    if (r < 0) {
+      cerr << "bench-write failed: " << cpp_strerror(-r) << std::endl;
+      return EXIT_FAILURE;
+    }
+    break;
   }
 
   return 0;
